import 'dart:math';

import 'package:flutter/material.dart';
import 'package:tencent_cloud_chat/data/theme/color/color_base.dart';
import 'package:tencent_cloud_chat/data/theme/text_style/text_style.dart';
import 'package:tencent_cloud_chat/models/tencent_cloud_chat_models.dart';
import 'package:tencent_cloud_chat/tencent_cloud_chat.dart';
import 'package:tencent_cloud_chat/utils/tencent_cloud_chat_utils.dart';
import 'package:tencent_cloud_chat_common/base/tencent_cloud_chat_theme_widget.dart';
import 'package:tencent_cloud_chat_message/tencent_cloud_chat_message_widgets/tencent_cloud_chat_message_item.dart';

class TencentCloudChatMessageCustom extends TencentCloudChatMessageItemBase {
  const TencentCloudChatMessageCustom({
    super.key,
    required super.message,
    required super.shouldBeHighlighted,
    required super.clearHighlightFunc,
    super.messageReceipt,
    required super.messageRowWidth,
    super.sendingMessageData,
    required super.renderOnMenuPreview,
    required super.inSelectMode,
    required super.onSelectMessage,
  });

  @override
  State<StatefulWidget> createState() => _TencentCloudChatMessageCustomState();
}

class _TencentCloudChatMessageCustomState extends TencentCloudChatMessageState<TencentCloudChatMessageCustom> {
  renderCustomMessage(TencentCloudChatThemeColors colorTheme, TencentCloudChatTextStyle textStyle) {
    bool isvotemessage = TencentCloudChatUtils.isVoteMessage(widget.message);
    if (isvotemessage) {
      if (TencentCloudChat.dataInstance.basic.hasPlugins(TencentCloudChatPluginNameEnum.poll)) {
        var plugin = TencentCloudChat.dataInstance.basic.getPlugin(TencentCloudChatPluginNameEnum.poll);
        if (plugin != null) {
          if (plugin.builder != null) {
            return plugin.builder!(widget.message);
          }
        }
      }
      return Container();
    }
    return Text(
      TencentCloudChatUtils.handleCustomMessage(widget.message),
      style: TextStyle(color: sentFromSelf ? colorTheme.selfMessageTextColor : colorTheme.othersMessageTextColor, fontSize: textStyle.messageBody),
    );
  }

  @override
  Widget defaultBuilder(BuildContext context) {
    final maxBubbleWidth = widget.messageRowWidth * 0.8;
    bool isvotemessage = TencentCloudChatUtils.isVoteMessage(widget.message);
    return TencentCloudChatThemeWidget(build: (context, colorTheme, textStyle) {
      if (isvotemessage) {
        return renderCustomMessage(colorTheme, textStyle);
      }
      return Container(
        padding: EdgeInsets.symmetric(horizontal: getWidth(10), vertical: getHeight(8)),
        decoration: BoxDecoration(
            color: showHighlightStatus ? colorTheme.info : (sentFromSelf ? colorTheme.selfMessageBubbleColor : colorTheme.othersMessageBubbleColor),
            border: Border.all(
              color: sentFromSelf ? colorTheme.selfMessageBubbleBorderColor : colorTheme.othersMessageBubbleBorderColor,
            ),
            borderRadius: BorderRadius.only(
              topLeft: Radius.circular(getSquareSize(16)),
              topRight: Radius.circular(getSquareSize(16)),
              bottomLeft: Radius.circular(getSquareSize(sentFromSelf ? 16 : 0)),
              bottomRight: Radius.circular(getSquareSize(sentFromSelf ? 0 : 16)),
            )),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.end,
          children: [
            ConstrainedBox(
              constraints: BoxConstraints(maxWidth: min(maxBubbleWidth * 0.9, maxBubbleWidth - getSquareSize(sentFromSelf ? 128 : 102))),
              child: renderCustomMessage(colorTheme, textStyle),
            ),
            SizedBox(
              width: getWidth(4),
            ),
            if (sentFromSelf) messageStatus(),
            messageTimeIndicator(),
          ],
        ),
      );
    });
  }
}
